@@ -7,11 +7,11 @@ impl<'a, T> Iterator for ArbitraryChunkMut<'a, T> {
 
     fn next(&mut self) -> Option<Self::Item> {
         let c = self.1.pop()?;
-        let slice = mem::replace(&mut self.0, &mut []);
-        if slice.is_empty() {
+        if self.0.is_empty() {
             return None;
         }
 
+        let slice = mem::replace(&mut self.0, &mut []);
         let (l, r) = slice.split_at_mut(c);
         self.0 = r;
 
